@@ -20,6 +20,7 @@
 
 #include "core_display.h"
 #include "core_commands2.h"
+#include "core_globals.h"
 #include "core_helpers.h"
 #include "core_main.h"
 #include "core_tables.h"
@@ -43,7 +44,7 @@
 #endif
 
 
-static char bigchars[130][5] =
+static const char bigchars[130][5] =
     {
         { 0x08, 0x08, 0x2a, 0x08, 0x08 },
         { 0x22, 0x14, 0x08, 0x14, 0x22 },
@@ -177,7 +178,7 @@ static char bigchars[130][5] =
         { 0x04, 0x08, 0x70, 0x08, 0x04 }
     };
 
-static char smallchars[329] =
+static const char smallchars[329] =
     {
         0x00, 0x00, 0x00,
         0x5c,
@@ -551,108 +552,136 @@ static int get_cat_index();
 
 
 bool persist_display() {
-    if (!shell_write_saved_state(catalogmenu_section, 5 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(catalogmenu_rows, 5 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(catalogmenu_row, 5 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(catalogmenu_item, 30 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(custommenu_length, 18 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(custommenu_label, 126))
-        return false;
+    for (int i = 0; i < 5; i++) {
+        if (!write_int(catalogmenu_section[i])) return false;
+        if (!write_int(catalogmenu_rows[i])) return false;
+        if (!write_int(catalogmenu_row[i])) return false;
+        for (int j = 0; j < 6; j++)
+            if (!write_int(catalogmenu_item[i][j])) return false;
+    }
+    for (int i = 0; i < 3; i++) {
+        for (int j = 0; j < 6; j++) {
+            if (!write_int(custommenu_length[i][j])) return false;
+            if (fwrite(custommenu_label[i][j], 1, 7, gfile) != 7) return false;
+        }
+    }
     for (int i = 0; i < 9; i++)
         if (!write_arg(progmenu_arg + i))
             return false;
-    if (!shell_write_saved_state(progmenu_is_gto, 9 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(progmenu_length, 6 * sizeof(int)))
-        return false;
-    if (!shell_write_saved_state(progmenu_label, 42))
-        return false;
-    if (!shell_write_saved_state(display, 272))
-        return false;
-    if (!shell_write_saved_state(&appmenu_exitcallback, sizeof(int)))
+    for (int i = 0; i < 9; i++)
+        if (!write_int(progmenu_is_gto[i])) return false;
+    for (int i = 0; i < 6; i++) {
+        if (!write_int(progmenu_length[i])) return false;
+        if (fwrite(progmenu_label[i], 1, 7, gfile) != 7) return false;
+    }
+    if (fwrite(display, 1, 272, gfile) != 272)
         return false;
+    if (!write_int(appmenu_exitcallback)) return false;
     return true;
 }
 
 bool unpersist_display(int version) {
-    int custommenu_cmd[3][6];
-    is_dirty = 0;
-    if (shell_read_saved_state(catalogmenu_section, 5 * sizeof(int))
-            != 5 * sizeof(int))
-        return false;
-    if (shell_read_saved_state(catalogmenu_rows, 5 * sizeof(int))
-            != 5 * sizeof(int))
-        return false;
-    if (shell_read_saved_state(catalogmenu_row, 5 * sizeof(int))
-            != 5 * sizeof(int))
-        return false;
-    if (shell_read_saved_state(catalogmenu_item, 30 * sizeof(int))
-            != 30 * sizeof(int))
-        return false;
+    if (state_is_portable) {
+        for (int i = 0; i < 5; i++) {
+            if (!read_int(&catalogmenu_section[i])) return false;
+            if (!read_int(&catalogmenu_rows[i])) return false;
+            if (!read_int(&catalogmenu_row[i])) return false;
+            for (int j = 0; j < 6; j++)
+                if (!read_int(&catalogmenu_item[i][j])) return false;
+        }
+        for (int i = 0; i < 3; i++) {
+            for (int j = 0; j < 6; j++) {
+                if (!read_int(&custommenu_length[i][j])) return false;
+                if (fread(custommenu_label[i][j], 1, 7, gfile) != 7) return false;
+            }
+        }
+        for (int i = 0; i < 9; i++)
+            if (!read_arg(progmenu_arg + i, false))
+                return false;
+        for (int i = 0; i < 9; i++)
+            if (!read_int(&progmenu_is_gto[i])) return false;
+        for (int i = 0; i < 6; i++) {
+            if (!read_int(&progmenu_length[i])) return false;
+            if (fread(progmenu_label[i], 1, 7, gfile) != 7) return false;
+        }
+        if (fread(display, 1, 272, gfile) != 272)
+            return false;
+        if (!read_int(&appmenu_exitcallback)) return false;
+    } else {
+        int custommenu_cmd[3][6];
+        is_dirty = 0;
+        if (fread(catalogmenu_section, 1, 5 * sizeof(int), gfile)
+                != 5 * sizeof(int))
+            return false;
+        if (fread(catalogmenu_rows, 1, 5 * sizeof(int), gfile)
+                != 5 * sizeof(int))
+            return false;
+        if (fread(catalogmenu_row, 1, 5 * sizeof(int), gfile)
+                != 5 * sizeof(int))
+            return false;
+        if (fread(catalogmenu_item, 1, 30 * sizeof(int), gfile)
+                != 30 * sizeof(int))
+            return false;
 
-    if (version < 7) {
-        /* In version 7, I removed the special handling
-         * of FCN catalog assignments (after discovering how
-         * the real HP-42S does it and realizing that, for perfect
-         * compatibility, I had to do it the same way).
-         */
-        if (shell_read_saved_state(custommenu_cmd, 18 * sizeof(int))
+        if (version < 7) {
+            /* In version 7, I removed the special handling
+             * of FCN catalog assignments (after discovering how
+             * the real HP-42S does it and realizing that, for perfect
+             * compatibility, I had to do it the same way).
+             */
+            if (fread(custommenu_cmd, 1, 18 * sizeof(int), gfile)
+                    != 18 * sizeof(int))
+                return false;
+        }
+        if (fread(custommenu_length, 1, 18 * sizeof(int), gfile)
                 != 18 * sizeof(int))
             return false;
-    }
-    if (shell_read_saved_state(custommenu_length, 18 * sizeof(int))
-            != 18 * sizeof(int))
-        return false;
-    if (shell_read_saved_state(custommenu_label, 126)
-            != 126)
-        return false;
-    if (version < 7) {
-        /* Starting with version 7, FCN catalog assignments are no longer
-         * handled specially; instead, commands with shortened key labels
-         * (e.g. ENTER/ENTR, ASSIGN/ASGN) are handled by using "meta"
-         * characters to indicate the disappearing positions. This is how
-         * the HP-42S does it, and since this can even affect programs
-         * (e.g. create the line STO "ENTER" by selecting ENTER from the
-         * function catalog, and the second E is encoded as meta-E in the
-         * program!), we have to do it the same way, for compatibility.
-         * I think my original approach was better, but such is life. :-)
-         */
-        int row, pos;
-        for (row = 0; row < 3; row++)
-            for (pos = 0; pos < 6; pos++) {
-                int cmd = custommenu_cmd[row][pos];
-                if (cmd != CMD_NONE) {
-                    const command_spec *cs = cmdlist(cmd);
-                    string_copy(custommenu_label[row][pos],
-                                &custommenu_length[row][pos],
-                                cs->name, cs->name_length);
-                }
-            }
-    }
-
-    for (int i = 0; i < 9; i++)
-        if (!read_arg(progmenu_arg + i, version < 9))
+        if (fread(custommenu_label, 1, 126, gfile)
+                != 126)
             return false;
-    if (shell_read_saved_state(progmenu_is_gto, 9 * sizeof(int))
-            != 9 * sizeof(int))
-        return false;
-    if (shell_read_saved_state(progmenu_length, 6 * sizeof(int))
-            != 6 * sizeof(int))
-        return false;
-    if (shell_read_saved_state(progmenu_label, 42)
-            != 42)
-        return false;
-    if (shell_read_saved_state(display, 272)
-            != 272)
-        return false;
-    if (shell_read_saved_state(&appmenu_exitcallback, sizeof(int))
-            != sizeof(int))
-        return false;
+        if (version < 7) {
+            /* Starting with version 7, FCN catalog assignments are no longer
+             * handled specially; instead, commands with shortened key labels
+             * (e.g. ENTER/ENTR, ASSIGN/ASGN) are handled by using "meta"
+             * characters to indicate the disappearing positions. This is how
+             * the HP-42S does it, and since this can even affect programs
+             * (e.g. create the line STO "ENTER" by selecting ENTER from the
+             * function catalog, and the second E is encoded as meta-E in the
+             * program!), we have to do it the same way, for compatibility.
+             * I think my original approach was better, but such is life. :-)
+             */
+            int row, pos;
+            for (row = 0; row < 3; row++)
+                for (pos = 0; pos < 6; pos++) {
+                    int cmd = custommenu_cmd[row][pos];
+                    if (cmd != CMD_NONE) {
+                        const command_spec *cs = cmdlist(cmd);
+                        string_copy(custommenu_label[row][pos],
+                                    &custommenu_length[row][pos],
+                                    cs->name, cs->name_length);
+                    }
+                }
+        }
+
+        for (int i = 0; i < 9; i++)
+            if (!read_arg(progmenu_arg + i, version < 9))
+                return false;
+        if (fread(progmenu_is_gto, 1, 9 * sizeof(int), gfile)
+                != 9 * sizeof(int))
+            return false;
+        if (fread(progmenu_length, 1, 6 * sizeof(int), gfile)
+                != 6 * sizeof(int))
+            return false;
+        if (fread(progmenu_label, 1, 42, gfile)
+                != 42)
+            return false;
+        if (fread(display, 1, 272, gfile)
+                != 272)
+            return false;
+        if (fread(&appmenu_exitcallback, 1, sizeof(int), gfile)
+                != sizeof(int))
+            return false;
+    }
     return true;
 }
 
@@ -844,16 +873,11 @@ void draw_char(int x, int y, char c) {
     mark_dirty(Y, X, Y + 8, X + 5);
 }
 
-void get_char(char *bits, char c) {
-    /* TODO - it should be possible to simply return bigchars[c],
-     * but that crashes on the Palm. Looks like a compiler bug.
-     */
-    int i;
+const char *get_char(char c) {
     unsigned char uc = (unsigned char) c;
     if (uc >= 130)
         uc -= 128;
-    for (i = 0; i < 5; i++)
-        bits[i] = bigchars[uc][i];
+    return bigchars[uc];
 }
 
 void draw_string(int x, int y, const char *s, int length) {
@@ -977,7 +1001,7 @@ static int prgmline2buf(char *buf, int len, int4 line, int highlight,
         string2buf(buf, len, &bufptr, ".END.", 5);
     } else if (cmd == CMD_NUMBER) {
         char *num = phloat2program(arg->val_d);
-        int numlen = strlen(num);
+        int numlen = (int) strlen(num);
         if (bufptr + numlen <= len) {
             memcpy(buf + bufptr, num, numlen);
             bufptr += numlen;
@@ -1654,12 +1678,15 @@ static void draw_catalog() {
             switch (type) {
                 case TYPE_REAL:
                 case TYPE_STRING:
-                    if (show_real) vcount++; break;
+                    if (show_real) vcount++;
+		    break;
                 case TYPE_COMPLEX:
-                    if (show_cpx) vcount++; break;
+                    if (show_cpx) vcount++;
+		    break;
                 case TYPE_REALMATRIX:
                 case TYPE_COMPLEXMATRIX:
-                    if (show_mat) vcount++; break;
+                    if (show_mat) vcount++;
+		    break;
             }
         }
         if (vcount == 0) {
